import argparse
import sys

# parser for protein generator
def get_args_score(mlist):

    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("--network=", type=str, required=False, dest='net', 
                        default='trunk_msa_v00', choices=mlist, help='network to use')
    
    parser.add_argument("--device=", type=str, required=False, dest='device', 
                        default='cpu', choices=['cpu','cuda'], help='device to use')

    parser.add_argument("--threads=", type=int, required=False, dest='nthreads', default=1,
                        help="number of cpu threads to use for device='cpu'")

    #parser.add_argument("-f", "--fasta-dir=", type=str, required=False, dest='fas_dir', 
    #                    default='', help='folder storing protein sequences in FASTA format')

    parser.add_argument("-p", "--pdb-dir=", type=str, required=False, dest='pdb_dir', 
                        help='folder storing protein structures in PDB format')

    parser.add_argument("-l", "--list=", type=str, required=False, dest='list', 
                        help='list of PDBs or FASTA-PDB pairs subject to evaluation')

    parser.add_argument("--ocsv=", type=str, required=False, dest='ocsv', 
                        help='save scoring output to a CSV files')

    parser.add_argument("--opdb=", type=str, required=False, dest='opdb', 
                        help='folder for saving PDB files')

    parser.add_argument("--onpz=", type=str, required=False, dest='onpz', 
                        help='folder for saving NPZ files')

    # trFold parameters
    parser.add_argument("-b", "--batch=", type=int, required=False, dest='batch', default=64, help='trFold batch size')
    parser.add_argument("-lr", "--learning-rate=", type=float, required=False, dest='lr', default=0.2, help='trFold learning rate')
    parser.add_argument("-n", "--nsteps=", type=int, required=False, dest='nsteps', default=100, help='trFold number of minimization steps')


    args = parser.parse_args()
    if args.pdb_dir is None and args.list is None:
        sys.exit('ERROR: Either --list or --pdb-dir must be provided.')

    return args

